package core.notevault;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.Menu;
import android.widget.Toast;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.navigation.fragment.NavHostFragment;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkManager;
import com.google.android.material.snackbar.Snackbar;
import com.google.android.material.navigation.NavigationView;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.AppBarConfiguration;
import androidx.navigation.ui.NavigationUI;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.appcompat.app.AppCompatActivity;
import core.notevault.data.*;
import core.notevault.databinding.ActivityMainBinding;
import core.notevault.sync.SyncWorker;
import core.notevault.sync.auth.AuthRepository;
import core.notevault.sync.auth.LoginCallback;
import core.notevault.ui.gallery.GalleryFragment;
import core.notevault.ui.gallery.detail.ConcertSongSelector;
import core.notevault.ui.gallery.editor.ConcertEditorDialog;
import core.notevault.ui.home.HomeFragment;
import core.notevault.ui.login.LoginCallBackImpl;
import core.notevault.ui.login.LoginDialogFragment;
import core.notevault.ui.login.RegisterCallback;
import core.notevault.ui.metadatadialog.MetaDataDialog;
import core.notevault.ui.metadatadialog.SongEditDialog;
import core.notevault.util.NoteSheetsUtil;

import java.io.*;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class MainActivity extends AppCompatActivity implements MetaDataDialog.OnMetadataListener,
        ConcertEditorDialog.OnConcertEditorListener, ConcertSongSelector.OnSongSelectedListener, LoginCallback, SongEditDialog.SongEditorListener {

    private AppBarConfiguration mAppBarConfiguration;
    private ActivityMainBinding binding;

    private MusicDatabase musicDB;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        setSupportActionBar(binding.appBarMain.toolbar);
        DrawerLayout drawer = binding.drawerLayout;
        NavigationView navigationView = binding.navView;
        // Passing each menu ID as a set of Ids because each
        // menu should be considered as top level destinations.
        mAppBarConfiguration = new AppBarConfiguration.Builder(
                R.id.nav_home, R.id.nav_gallery, R.id.nav_slideshow)
                .setOpenableLayout(drawer)
                .build();
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_main);
        NavigationUI.setupActionBarWithNavController(this, navController, mAppBarConfiguration);
        NavigationUI.setupWithNavController(navigationView, navController);

        setupLoginButton();
        musicDB = MusicDatabase.getDatabase(this);
        scheduleSync();
    }

    public void scheduleSync() {
        // Setze eine Uhrzeit für die Synchronisation
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY,11);
        calendar.set(Calendar.MINUTE, 32);
        calendar.set(Calendar.SECOND, 0);

        long triggerTime = calendar.getTimeInMillis(); // Zeitstempel für den Trigger

        // Erstelle die OneTimeWorkRequest für den SyncWorker
        OneTimeWorkRequest syncRequest = new OneTimeWorkRequest.Builder(SyncWorker.class)
                .setInitialDelay(triggerTime - System.currentTimeMillis(), TimeUnit.MILLISECONDS) // Verzögerung bis zur Ausführung
                .build();

        // Planen des Workers mit WorkManager
        WorkManager.getInstance(getBaseContext()).enqueue(syncRequest);
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        MenuItem loginItem = menu.findItem(R.id.action_login);
        if (isLoggedIn()) {
            loginItem.setIcon(R.drawable.logout); // Setze das Logout-Symbol
        } else {
            loginItem.setIcon(R.drawable.login); // Setze das Login-Symbol
        }
        return super.onPrepareOptionsMenu(menu);
    }

    public void updateLoginButton() {
        invalidateOptionsMenu(); // Menü neu zeichnen
    }

    private void setupLoginButton() {
        binding.appBarMain.toolbar.setOnMenuItemClickListener(item -> {
            if (item.getItemId() == R.id.action_login) {
                if (isLoggedIn()) {
                    performLogout(); // Logout-Logik aufrufen
                } else {
                    showLoginDialog(); // Zeige das Login-Fenster an
                }
                return true;
            }
            return false;
        });
    }

    private boolean isLoggedIn() {
        AuthRepository authRepository = new AuthRepository(this);
        return !TextUtils.isEmpty(authRepository.getToken());
    }

    private void showLoginDialog() {
        LoginDialogFragment loginDialogFragment = LoginDialogFragment.newInstance(new LoginCallBackImpl(this), new RegisterCallback(this));
        loginDialogFragment.show(getSupportFragmentManager(), "login");
    }

    private void performLogout() {
        // Füge hier deine Logout-Logik hinzu
        AuthRepository authRepository = new AuthRepository(this);

        authRepository.logout(); // Implementiere die Token-Löschung
        updateLoginButton();
    }



    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if(id == R.id.action_login) {
            openLoginDialog();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void openLoginDialog() {
        LoginDialogFragment loginDialogFragment = new LoginDialogFragment();
        loginDialogFragment.show(getSupportFragmentManager(), "LOGIN_TAG");
    }

    @Override
    public boolean onSupportNavigateUp() {
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_main);
        return NavigationUI.navigateUp(navController, mAppBarConfiguration)
                || super.onSupportNavigateUp();
    }

    @Override
    public void onMetadataEntered(Uri[] uris, String title, String composer, int year, String genre) {
        new Thread(() -> {
            MusicNote musicNote = new MusicNote(title, composer, year, genre);
            long musicNoteID = musicDB.musicNoteDao().insert(musicNote);
            musicNote.setMusicNoteId(musicNoteID);

            Log.d("MainActivity", "MusicNoteID of inserted song: " + musicNoteID);
            Log.d("MainActivity", "MusicNoteID of referenced song: " + musicNote.getMusicNoteId());
            for(Uri uri: uris) {
                String filePath = saveImageInternally(uri);
                NoteSheet noteSheet = new NoteSheet(musicNoteID, filePath);
                musicDB.musicNoteDao().insertNoteSheet(noteSheet);
            }

            runOnUiThread(() -> {
                NavHostFragment navHostFragment = (NavHostFragment) getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment_content_main);
                Fragment currentFragment = navHostFragment != null ? navHostFragment.getChildFragmentManager().getFragments().get(0) : null;

                if(currentFragment instanceof HomeFragment) {
                    HomeFragment homeFragment = (HomeFragment) currentFragment;
                    homeFragment.addSong(musicNote);
                }
            });
        }).start();
    }

    // Speichere eine Kopie des Bilds im internen Speicher
    private String saveImageInternally(Uri uri) {
        try (InputStream inputStream = getContentResolver().openInputStream(uri)) {
            File imageFile = new File(getFilesDir(), "saved_image_" + System.currentTimeMillis() + ".jpg");
            try (OutputStream outputStream = Files.newOutputStream(imageFile.toPath())) {
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            }
            return imageFile.getAbsolutePath(); // Pfad speichern, um später darauf zuzugreifen
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public void onConcertEditorClosed(String concertTitle, String concertDate, int concertID) {
        new Thread(() -> {
            if(concertID < 0) {
                Concert concert = new Concert(concertTitle, concertDate);
                Log.d("ConcertEditor", "Saved Concert: " + concertTitle + " on " + concertDate);
                musicDB.musicNoteDao().insertConcert(concert);

                runOnUiThread(() -> {
                    NavHostFragment navHostFragment = (NavHostFragment) getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment_content_main);
                    Fragment currentFragment = navHostFragment != null ? navHostFragment.getChildFragmentManager().getFragments().get(0) : null;

                    Log.d("MainActivity", "Test for GalleryFragment");
                    Log.d("MainActivity", "Current Fragment: " + currentFragment.getClass().getSimpleName());
                    if(currentFragment instanceof GalleryFragment) {
                        Log.d("MainActivity", "GalleryFragment Found");
                        GalleryFragment galleryFragment = (GalleryFragment) currentFragment;
                        galleryFragment.addConcert(concert);
                    } else {
                        Log.d("MainActivity", "GalleryFragment Not Found");
                    }
                });
            } else {
                Concert concert = new Concert(concertID, concertTitle, concertDate);
                musicDB.musicNoteDao().updateConcert(concert);
                runOnUiThread(() -> {
                    NavHostFragment navHostFragment = (NavHostFragment) getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment_content_main);
                    Fragment currentFragment = navHostFragment != null ? navHostFragment.getChildFragmentManager().getFragments().get(0) : null;

                    Log.d("MainActivity", "Test for GalleryFragment");
                    Log.d("MainActivity", "Current Fragment: " + currentFragment.getClass().getSimpleName());
                    if(currentFragment instanceof GalleryFragment) {
                        Log.d("MainActivity", "GalleryFragment Found");
                        GalleryFragment galleryFragment = (GalleryFragment) currentFragment;
                        galleryFragment.updateConcert(concert);
                    } else {
                        Log.d("MainActivity", "GalleryFragment Not Found");
                    }
                });
            }

        }).start();
    }

    @Override
    public void onSongsSelected(List<MusicNote> songs, int concertID) {
        Log.d("MainActivity", "Inserted Songs: " + songs.size());
        new Thread(() -> {
            for(MusicNote musicNote : songs) {
                ConcertSong concertSong = new ConcertSong(musicNote.getMusicNoteId(), concertID);
                musicDB.musicNoteDao().insertConcertSong(concertSong);
                Log.d("MainActivity", "Insert Song: " + musicNote.getTitle());
            }
        }).start();
    }


    @Override
    public void onSuccess() {
        updateLoginButton();
    }

    @Override
    public void onError(String error) {
        Toast.makeText(this, error, Toast.LENGTH_LONG).show();
        Log.d("LoginError", error);
    }

    @Override
    public void onSongEdited(MusicNote updatedSong) {
        new Thread(() -> {
            musicDB.musicNoteDao().updateSong(updatedSong);
            Log.d("MainActivity", "Updated Song: " + updatedSong.getTitle());
        }).start();
        NavHostFragment navHostFragment = (NavHostFragment) getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment_content_main);
        Fragment currentFragment = navHostFragment != null ? navHostFragment.getChildFragmentManager().getFragments().get(0) : null;

        if(currentFragment instanceof HomeFragment) {
            HomeFragment homeFragment = (HomeFragment) currentFragment;
            homeFragment.updateSong(updatedSong);
        } else if(currentFragment instanceof GalleryFragment) {
            GalleryFragment galleryFragment = (GalleryFragment) currentFragment;

        }
    }
}